' This has been adapted from the zog_debug.spin program that comes with zog.
' It serves a similar role to it, but for the 6502 emulator

'#define SINGLE_STEP


CON
    _clkmode = xtal1 + pll16x         ' enable external clock and pll times 16
    _xinfreq = 5_000_000              ' set frequency to 5 MHZ

    ' user baud rate
    baud            = 38400

    AUDIO_PIN           = 24
DAT                     org 0
vm_memory              byte ' Force vm_memory to be BYTE type.
filename byte "Dawn.sid",0

CON
vm_memory_size = (64 * 1024)                            'The size of memory area


OBJ
  io : "c3io"
  mos6502  : "c3io_6502"
  sid   : "SIDcog"
  ser  : "FullDuplexSerialPlus"
  sd   : "c3io_fsrw"                          'SD Software used in FemtoBASIC

VAR

  long  emu_mode                ' these variables have to be in this order
  long  emu_status
  long  reg6502_acc
  long  reg6502_xreg
  long  reg6502_yreg
  long  reg6502_status
  long  reg6502_stackptr
  long  reg6502_pc
  long  iocmd
  long  iolock


  'Misc vars
  long cog
  long ioControl[2]               'SD parameters
  long framep
  word err                        'Error code return
  long step_count
  byte diskbuff[512]



  ' SID Vars
  byte sidName[32]
  byte sidAuthor[32]
  byte sidCopyright[32]

  byte max_sub_songs
  byte sub_song_start
  byte play_speed


  word load_addr
  word init_addr
  word play_addr

PUB start : okay | n, i, sidbase
  ser.start(31, 30, %0000, 38400) 'Start the debug Terminal
  ser.str(string("c3io-6502 v0.1"))

  crlf
  waitcnt(cnt+80_000_000)                               'Give serial terminal window time

  ser.str (string("Starting IO/Cache driver...",))
  err := io.start(@ioControl)                         'Start the SD routines
  ser.hex(err, 8)
  crlf

  waitcnt(cnt+80_000_000)                               'Give serial terminal window time

  ser.str (string("Mounting SD...",))
  err := \sd.start
  err := \sd.mount  'Mount the SD
  ser.hex(err, 8)
  crlf

  ser.str (string("Starting SIGCog...",))
  crlf
  sidbase := sid.start(AUDIO_PIN,0)

  ' set volume high
  sid.setVolume(15)

  ser.str (string("Starting 6502 Cog...",))
  crlf
  mos6502.setCpuMode(0)          ' CPU starts frozen as cog launches
  mos6502.reset(-1)              ' reset CPU state
  mos6502.launch(sidbase)  ' start the cog

  load_sidfile

  ser.str (string("load_addr: "))
  ser.hex(load_addr, 8)
  crlf
  ser.str (string("init_addr: "))
  ser.hex(init_addr, 8)
  crlf
  ser.str (string("play_addr: "))
  ser.hex(play_addr, 8)
  crlf

  crlf
  ser.str(string("Running the sidfile init!"))
  crlf
  mos6502.JSR(init_addr, sub_song_start)

  ser.str(string("Beginning play of SIDFile!"))
  crlf

  play_loop

pub play_loop | waitDelta, waitTime, tickcnt
  'Wait 1/50 sec between updates (PAL VBL), or twice that between calls to play_addr
  if play_speed == 0
    waitDelta := 80_000_000/50
  else
    waitDelta := 80_000_000/100   '' double speed
'  waitTime := cnt + waitDelta
  waitTime := cnt + 10_000_000


  repeat
    mos6502.JSR(play_addr, 0)
'    print_sid_regs
'    waitcnt( waitTime )
'    waitTime := waitDelta + cnt
'    waitcnt( waitDelta + cnt )

'    sid.updateRegisters(@sidRegisters)
    '@buffer[24]:= (buffer[24]&$F0) | (volume&$0F)
    'for now, just max the volume


    if tickcnt++ == 50
      tickcnt := 0
      ser.tx(".")


pub load_sidfile | data_file_offset, i, ramaddr, bytes, sbytes

  ser.str (string("Loading: ",))
  ser.str (@filename)
  ser.str (string($d,$a))
  err := \sd.popen(@filename, "r")
  crlf
  ser.hex(err, 8)
  crlf

  sd.seek($16)
  sd.pread(@sidName, 32)
  ser.str (string("Name:      "))
  ser.str (@sidName)
  crlf
  sd.seek($36)
  sd.pread(@sidAuthor, 32)
  ser.str (string("Authour:   "))
  ser.str (@sidAuthor)
  crlf
  sd.seek($56)
  sd.pread(@sidCopyright, 32)
  ser.str (string("Copyright: "))
  ser.str (@sidCopyright)
  crlf

  sd.seek(7)
  data_file_offset := sd.pgetc           ' pos 7

  load_addr := sd.pgetc << 8             ' pos 8
  load_addr |= sd.pgetc                  ' pos 9

  init_addr := sd.pgetc << 8             ' pos 10
  init_addr |= sd.pgetc                  ' pos 11

  play_addr := sd.pgetc << 8             ' pos 12
  play_addr |= sd.pgetc                  ' pos 13

  sd.seek($f)
  max_sub_songs := sd.pgetc - 1
  sd.seek($11)
  sub_song_start := sd.pgetc - 1

  sd.seek($15)
  play_speed := sd.pgetc

  sd.seek(data_file_offset)
  load_addr := sd.pgetc
  load_addr |= sd.pgetc << 8

  ser.str (string($d,$a,"Clearing RAM: ",))
  repeat i from 0 to vm_memory_size-1
    mos6502.poke(i, 0)
    ifnot i & ($800-1)
      ser.tx(".")
  crlf

  ser.str (string($d,$a,"Loading SID File to RAM: ",))

  sd.seek(data_file_offset+2)
  ramaddr := load_addr
  bytes := 1
  sbytes~
  repeat while bytes > 0
    bytes := sd.pread(@diskbuff, 512)
    sbytes+= bytes
'    err := sd.stopSDCard                                'stop SD card, but do not stop cog
    repeat i from 0 to bytes - 1
      mos6502.poke(ramaddr, diskbuff[i])                'To virtual memory.
      ramaddr++

  ser.str(string(" "))
  ser.dec(sbytes)
  ser.str(string(" Bytes Loaded."))
  crlf
  ser.str (string("Done",))
  crlf

'  err := sd.stopSDCard                                  'stop SD card, but do not stop cog
  CheckError(err)

  ' deal with play_addr that needs init_addr to be called first
  if play_addr == 0
    ' so now we have frozen 6502, about to JSR into the init routine, which
    ' will eventually try to RTS, resulting it popping a PC of $0000 off.
    ' we will step the processor through to this state, and then move along

    mos6502.JSR(init_addr, 0)
    play_addr := mos6502.peek($0315) << 8
    play_addr |= mos6502.peek($0314)

  return load_addr

PRI print_regs | i, p, op
  'Print the 6502 registers
  ser.str(string("0x"))
  ser.hex(mos6502.getPC, 4)
  ser.tx($20)
  ser.str(string("0x"))
  op := mos6502.peek(mos6502.getPC)                  'XOR here is an endianess fix.
  ser.hex(op, 2)
  ser.tx($20)
  ser.str(string("0x"))
  ser.hex(mos6502.getA, 2)
  ser.tx($20)
  ser.str(string("0x"))
  ser.hex(mos6502.getX, 2)
  ser.tx($20)
  ser.str(string("0x"))
  ser.hex(mos6502.getY, 2)
  ser.tx($20)
  ser.str(string("0x"))
  ser.hex(mos6502.getStatus, 2)
  ser.tx($20)
  ser.str(string("0x"))
  ser.hex(mos6502.getStackPtr, 2)
  ser.tx($20)
  ser.str(string("0x"))
  ser.hex(mos6502.getCpuMode, 2)
  ser.tx($20)
  ser.str(string("0x"))
  ser.hex(mos6502.getCpuStatus, 2)

  crlf
{
  case op
    $00:
      ser.str(string("BREAKPOINT"))
      crlf
    $01,$03,$0E,$0F:
      ser.str(string("ILLEGAL OP"))
      crlf
    $20,$21:
      ser.str(string("NOT_IMPLEMENTED"))
      crlf
}
'  p := 0 - 8*4
'  repeat i from 0 to 7
'    ser.hex(p + 4*i, 8)
'    ser.tx($20)
'    ser.hex(LONG[@vm_memory[(p {& vm_memory_mask})+ 4*i]], 8)
'    crlf
'  crlf

PRI crlf
  ser.tx(13)
  ser.tx(10)

PRI CheckError(r)
  if r < 0
    ser.str(string("Error: "))
    ser.hex(r,2)
    ser.str(string(" HALTED."))
    crlf
'    sd.stop
    repeat                                             '<== LOOP HERE

'---------------------------------------------------------------------------------------------------------

'---------------------------------------------------------------------------------------------------------
'The End.

